# Copyright (C) 2009-2010 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

# -*- coding: utf-8 -*-
"""Cluster controller module"""

# turbogears imports
from tg import request, expose, redirect, flash, app_globals, tmpl_context

# third party imports
from repoze.what.predicates import not_anonymous
from pylons.i18n import ugettext as _

# project specific imports
from luci.lib.base import BaseController
from luci.lib.db_helpers import get_cluster_list, get_model_for_cluster, get_status_for_cluster, db_remove_cluster, get_agent_for_cluster, get_cluster_db_obj
import luci.lib.ricci_helpers as rh
import luci.widget_validators.validate_cluster_prop as vcp
from luci.widget_validators.validate_create_cluster_form import validate_create_cluster_form, validate_node_add_form
from luci.widget_validators.validate_fence import validateNewFenceDevice, validateFenceDevice
from luci.widget_validators.validate_resource import validate_clusvc_form, validate_resource_form
from luci.lib.ClusterConf.Method import Method
from luci.widget_validators.validate_fence import validate_fenceinstance

import logging
log = logging.getLogger(__name__)

data = app_globals.data

class ClusterController(BaseController):
    allow_only = not_anonymous()
    tmpl_context.show_sidebar = True

    @expose('luci.templates.cluster_list')
    def index(self):
        tmpl_context.manage_clusters = True
        tmpl_context.show_sidebar = True
        return dict(page='cluster', clusterpage='clusterpage')

    @expose('luci.templates.cluster_list')
    def create_cmd(self, command=None, **kw):
        validate_create_cluster_form(self, **kw)
        return dict(use_referrer=False, page='cluster', clusterpage='clusterpage')

    @expose('luci.templates.cluster_list')
    def delete_cmd(self, command=None, **kw):
        cur_list = []
        if 'MultiAction' in kw:
            command = kw['MultiAction']
            for key, element in kw.items():
                if element == "on":
                     db_remove_cluster(unicode(key))
        redirect('/cluster/')

    @expose()
    def lookup(self, name, *args):
        cluster_list = get_cluster_list() or []
        for i in cluster_list:
            if i.name == name:
                icc = IndividualClusterController(name, data)
                return icc, args
        cc = ClusterController
        #flash(("No cluster named \"%s\" is managed by luci") % name,
        #    status='error')
        redirect('/cluster/')
        return (cc, args)

class IndividualClusterController(BaseController):
    def __init__(self, name, iccdata, nodename=None):
        self.agent  = None
        self.model  = None
        self.status = None
        self.version = None

        self.name = name
        self.data = iccdata
        self.nodename = nodename

        identity = request.environ.get('repoze.who.identity')
        self.username = identity['repoze.who.userid']

        tmpl_context.show_sidebar = True
        tmpl_context.cluster_name = self.name
        tmpl_context.cluster_url = '/cluster/%s/' % self.name
        tmpl_context.cluster = self

    @expose("luci.templates.node")
    def default(self):
        db = get_cluster_db_obj(self.name)

        return dict(page='nodes', name=self.nodename, base_url='/nodes', nodes=db.nodes)

    def get_agent(self):
        if not self.agent:
            try:
                self.agent = get_agent_for_cluster(self.name)
            except:
                self.agent = None
        return self.agent

    def get_model(self):
        if not self.model:
            self.model = get_model_for_cluster(self.name, self.get_agent())
        return self.model

    def get_status(self):
        if not self.status:
            self.status = get_status_for_cluster(self.name, self.get_agent())
        return self.status

    def get_version(self):
        if not self.version:
            self.version = app_globals.DEFAULT_CLUSTER_VERSION
            m = self.get_model()
            if m:
                self.version = m.getClusterVersion()

        return self.version

    @expose("luci.templates.node")
    def nodes(self):
        db = get_cluster_db_obj(self.name)
        return dict(use_referrer=False, page='nodes', name=self.nodename, base_url='/nodes', nodes=db.nodes)

    @expose("luci.templates.node")
    def add_nodes_cmd(self, command=None, **kw):
        self.get_model()
        db = get_cluster_db_obj(self.name)
        vret = validate_node_add_form(self.model, db, **kw)
        redirect('%s%s' % (tmpl_context.cluster_url, 'nodes'))

    @expose("luci.templates.node")
    def nodes_fence_cmd(self, command=None, **kw):
        self.get_model()
        if not self.model:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        if kw.get("method_to_remove"):
            node = self.model.retrieveNodeByName(kw.get("node"))
            methodname = kw.get("method_to_remove")
            for child in node.getFenceNode().getChildren():
                if child.getName() == methodname:
                    node.getFenceNode().removeChild(child)
                    break
            self.model.setModified(True)
            rh.update_cluster_conf(self.model)

        if kw.get("fenceinst_to_remove") and kw.get("fenceinst_to_remove_method"):
            node = self.model.retrieveNodeByName(kw.get("node"))
            fenceinst = kw.get("fenceinst_to_remove")
            methodname = kw.get("fenceinst_to_remove_method")
            for child in node.getFenceNode().getChildren():
                if child.getName() == methodname:
                    child.removeChild(child.children[int(fenceinst)-1])
            self.model.setModified(True)
            rh.update_cluster_conf(self.model)

        if kw.get("moveup_method"):
            node = self.model.retrieveNodeByName(kw.get("node"))
            methodname = kw.get("moveup_method")
            levels = node.getFenceLevels()
            for level in levels:
                if level.getAttribute("name") == methodname:
                    position = levels.index(level)
                    if position > 0:
                        levels.pop(position)
                        levels.insert(position-1,level)
                        self.model.setModified(True)
                        rh.update_cluster_conf(self.model)
                    break

        if kw.get("movedown_method"):
            node = self.model.retrieveNodeByName(kw.get("node"))
            methodname = kw.get("movedown_method")
            levels = node.getFenceLevels()
            for level in levels:
                if level.getAttribute("name") == methodname:
                    position = levels.index(level)
                    if position < len(levels) - 1:
                        levels.pop(position)
                        levels.insert(position+1,level)
                        self.model.setModified(True)
                        rh.update_cluster_conf(self.model)
                    break

        if command == "AddFence":
            node = self.model.retrieveNodeByName(kw.get("node"))
            levels = node.getFenceLevels()
            fence_method = Method()
            fence_method.addAttribute('name', '1')
            if len(levels) > 0:
                methodname = kw['methodname']
                fence_methods = node.getFenceLevels()
                for i in fence_methods:
                    if i.getAttribute("name") == methodname:
                        fence_method = i
                        break

            parent_fencedev = kw['parent_fencedev'].replace('fd_', '', 1)

            retcode, retobj = validate_fenceinstance(parent_fencedev, kw['fence_type'], **kw)
            if retcode is True:
                fence_method.addChild(retobj)
                if len(levels) == 0:
                    fence_node = node.getFenceNode()
                    fence_node.addChild(fence_method)
                flash(_('Updating fence settings'))
                self.model.setModified(True)
                rh.update_cluster_conf(self.model)
            else:
                msgs = retobj
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")

        if command == "EditFence":
            node = self.model.retrieveNodeByName(kw.get("node"))
            instance_id = kw.get("instance_id")
            (method_num,sep,fence_instance_id) = instance_id.partition('-')

            # Create the fence device, and if it succeeds replace the old one with the new one
            retcode, retobj = validate_fenceinstance(kw["fencedev"], kw['fence_type'], **kw)
            if retcode is True:
                node.getFenceNode().children[int(method_num)].children[int(fence_instance_id)] = retobj
                self.model.setModified(True)
                rh.update_cluster_conf(self.model)

        if command == "AddFenceMethod":
            node = self.model.retrieveNodeByName(kw.get("node"))
            methodname = kw.get("newmethodname")
            method = Method()
            method.addAttribute("name",methodname)
            node.getFenceNode().addChild(method)
            self.model.setModified(True)
            rh.update_cluster_conf(self.model)

        redirect('%s%s' % (tmpl_context.cluster_url, kw.get('node')))
        
    # This processes all of the commands that we can apply to a node
    @expose("luci.templates.node")
    def nodes_cmd(self, command=None, **kw):
        self.get_model()
        if not self.model:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]
        if len(cur_list) < 1:
            flash(_('No nodes were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)

        if command == 'Reboot':
            log.info('User "%s" rebooted "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Rebooting %s") % ', '.join(cur_list),
                status='info')
            rh.cluster_node_reboot(self.name, cur_list)
        elif command == 'Join Cluster':
            log.info('User "%s" started nodes "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Starting nodes: %s") % ', '.join(cur_list),
                status='info')
            rh.cluster_node_start(self.name, cur_list)
        elif command == 'Leave Cluster':
            log.info('User "%s" stopped nodes "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Stopping nodes: %s") % ', '.join(cur_list),
                status='info')
            rh.cluster_node_stop(self.name, cur_list)
        elif command == 'Delete':
            log.info('User "%s" deleted nodes "%s" from cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Deleting nodes: %s") % ', '.join(cur_list),
                status='info')
            rh.cluster_node_delete(self.name, self.model, cur_list)
        else:
            log.error('User "%s" submitted unknown command "%s" for nodes "%s" from cluster "%s"' % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for nodes "%s"')
                    % (command, ', '.join(cur_list)),
                status='error')
        redirect(tmpl_context.cluster_url + kw.get('name'))

    @expose("luci.templates.resource")
    def resources(self, *args):
        base_url = '/cluster/%s/resources' % self.name
        resources_cmd = '/cluster/%s/resources_cmd' % self.name

        if (len(args) >= 1):
            resourcename = '/'.join(args)
        else:
            resourcename = None
        return dict(page='nodes', name=resourcename, base_url=base_url, resources_cmd=resources_cmd)

    @expose("luci.templates.resource")
    def resources_cmd(self, command=None, **kw):
        self.get_model()
        tmpl_context.cluster_url = '/cluster/%s/resources' % self.name
        
        if not self.model:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]

        if len(cur_list) < 1 and command not in ('Create', 'Edit'):
            flash(_('No resources were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)
        if command == 'Delete':
            errors = []
            res_ptr = self.model.getResourcesPtr()
            for i in cur_list:
                try:
                    cur_res = self.model.getResourceByName(i)
                    if cur_res.getRefcount() > 1:
                        errors.append(_('Global resource "%s" is in use so cannot be removed') % i)
                    else:
                        res_ptr.removeChild(cur_res)
                except:
                    pass
            if len(errors) != 0:
                flash(_('%s') % ', '.join(errors), status='error')
            else:
                log.info('User "%s" deleted resource "%s" in cluster "%s"'
                    % (self.username, ', '.join(cur_list), self.name))
                flash(_("Deleting resources %s") % ', '.join(cur_list),
                    status='info')

                self.model.setModified()
                rh.update_cluster_conf(self.model)
        elif command in ("Create", "Edit"):
            if command == 'Create':
                cur_action = _('Created')
            else:
                cur_action = _('Edited')
            res_type = kw.get('type', '')
            if res_type == 'ip':
                # The whole address parameter has to be constructed
                # from its parts ('address_nominal', 'address_mask') first.
                address = kw.get('address_nominal', '').strip()
                if len(kw.get('address_mask', '')) > 0:
                    mask = kw.get('address_mask', '').strip()
                    if not mask.startswith('/'):
                        address += '/'
                    address += mask
                kw['address'] = address
                kw.pop('address_nominal', None)
                kw.pop('address_mask', None)
            vret = validate_resource_form(self.model, **kw)
            if vret[0] is True:
                # For IP resources there is no name, just an IP Address
                # and to make IP addresses work you need to add a .html
                if res_type == 'ip':
                    res_name = kw['address']
                    redir_fmt = '%s/%s.html'
                else:
                    res_name = kw.get('resourcename')
                    redir_fmt = '%s/%s'
                log.info('User "%s" %s global resource "%s" in cluster "%s"'
                    % (self.username, cur_action, res_name, self.name))
                flash(_('%s global resource "%s"') % (cur_action, res_name))
                self.model.setModified(True)
                rh.update_cluster_conf(self.model)
                redirect(redir_fmt % (tmpl_context.cluster_url, res_name))
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url)
        else:
            log.error('User "%s" submitted unknown command "%s" for resource "%s" from cluster "%s"' % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for resources "%s"')
                % (command, ', '.join(cur_list)),
                    status='error')
        redirect(tmpl_context.cluster_url)

    @expose("luci.templates.service")
    def services(self, *args):
        base_url = '/cluster/%s/services' % self.name
        services_cmd = '/cluster/%s/services_cmd' % self.name

        if (len(args) == 1):
            servicename = args[0]
        else:
            servicename = None
        return dict(page='nodes', name=servicename, base_url=base_url, services_cmd=services_cmd)

    @expose("luci.templates.service")
    def services_cmd(self, command=None, **kw):
        self.get_model()
        tmpl_context.cluster_url = '/cluster/%s/services' % self.name

        if not self.model:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)
        
        preferred_node = kw.get('preferred_node')

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]

        if len(cur_list) < 1 and command not in ('Create', 'Edit'):
            flash(_('No services were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)

        if command == 'Delete':
            log.info('User "%s" deleted service "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Deleting services %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                self.model.deleteService(i)

            self.model.setModified()
            rh.update_cluster_conf(self.model)
            redirect(tmpl_context.cluster_url)
        if command == 'Start':
            log.info('User "%s" started service "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Starting services %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                rh.cluster_svc_start(self.name, i, preferred_node)
        elif command == 'Disable':
            log.info('User "%s" disabled service "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Disabling services %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                rh.cluster_svc_disable(self.name, i)
        elif command == 'Restart':
            log.info('User "%s" restarted service "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_("Restarting services %s") % ', '.join(cur_list),
                status='info')
            for i in cur_list:
                rh.cluster_svc_restart(self.name, i)
        elif command in ('Create', 'Edit'):
            if command == 'Create':
                cur_action = _('Created')
            else:
                cur_action = _('Edited')
            vret = validate_clusvc_form(self.model, **kw)
            kw['name'] = ''
            if vret[0] is True:
                svc_name = kw.get('svc_name')
                log.info('User "%s" %s cluster service "%s" in cluster "%s"'
                    % (self.username, cur_action, svc_name, self.name))
                flash(_('%s cluster service "%s"') % (cur_action, svc_name))
                self.model.setModified(True)
                rh.update_cluster_conf(self.model)
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url)
        else:
            log.error('User "%s" submitted unknown command "%s" for service "%s" from cluster "%s"' % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for services "%s"')
                    % (command, ', '.join(cur_list)),
                status='error')
        if len(cur_list) != 1:
            redirect(tmpl_context.cluster_url)
        else:
            redirect(tmpl_context.cluster_url + '/' + cur_list[0])

    @expose("luci.templates.failover")
    def failovers(self, *args):
        failovers_cmd = '/cluster/%s/failovers_cmd' % self.name

        if len(args) == 1:
            failovername = args[0]
        else:
            failovername = None
        return dict(page='nodes', name=failovername, base_url = '/cluster/' + self.name + '/failovers', failovers_cmd=failovers_cmd)

    @expose("luci.templates.failover")
    def failovers_cmd(self, command=None, **kw):
        self.get_model()
        tmpl_context.cluster_url = '/cluster/%s/failovers' % self.name

        if not self.model:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]

        if len(cur_list) < 1 and command != 'create':
            flash(_('No failover domains were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)

        if command == 'Delete':
            log.info('User "%s" deleted failover domains "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_('Deleting failover domains %s') % ', '.join(cur_list))
            for i in cur_list:
                self.model.deleteFailoverDomain(i)
            self.model.setModified(True)
            rh.update_cluster_conf(self.model)

            redirect(tmpl_context.cluster_url)

        # Delete is the only command that can be applied to multiple
        # failover domains
        if len(cur_list) > 1:
            flash(_('The settings of exactly one failover domain may be updated at one time'),
                status='error')
            redirect(tmpl_context.cluster_url)
        
        if command == 'update_settings':
            vret = vcp.validate_fdom_prop_settings_form(self.model, **kw)
            if vret[0] is True:
                log.info('User "%s" updated the settings of failover domain "%s" in cluster "%s"'
                    % (self.username, cur_list[0], self.name))
                flash(_('Updating settings for failover domain "%s"') % cur_list[0])
                self.model.setModified(True)
                rh.update_cluster_conf(self.model)
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url + '/' + kw.get('name'))
        elif command == 'update_properties':
            vret = vcp.validate_fdom_prop_form(self.model, **kw)
            if vret[0] is True:
                log.info('User "%s" updated the properties of failover domain "%s" in cluster "%s"'
                    % (self.username, cur_list[0], self.name))
                flash(_('Updating properties for failover domain "%s"') % cur_list[0])
                self.model.setModified(True)
                rh.update_cluster_conf(self.model)
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url + '/' + kw.get('name'))
        elif command == 'create':
            vret = vcp.validate_fdom_create_form(self.model, **kw)
            if vret[0] is True:
                log.info('User "%s" created failover domain "%s" in cluster "%s"'
                    % (self.username, kw.get('fdom_name'), self.name))
                flash(_('Creating failover domain "%s"') % kw.get('fdom_name'))

                self.model.setModified(True)
                rh.update_cluster_conf(self.model)
            else:
                msgs = vret[1].get('errors')
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
            redirect(tmpl_context.cluster_url)
        else:
            log.error('User "%s" submitted unknown command "%s" for failover domains "%s" from cluster "%s"'
                % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for failover domains "%s"')
                    % (command, ', '.join(cur_list)),
                status='error')
        redirect(tmpl_context.cluster_url)

    @expose("luci.templates.fence")
    def fences(self, *args):
        fences_cmd = '/cluster/%s/fences_cmd' % self.name

        if len(args) == 1:
            fencename = args[0]
        else:
            fencename = None
        return dict(page='nodes', name=fencename, base_url='/cluster/' + self.name + '/fences', fences_cmd=fences_cmd)

    @expose("luci.templates.fence")
    def fences_cmd(self, command=None, **kw):
        self.get_model()
        tmpl_context.cluster_url = '/cluster/%s/fences' % self.name

        if not self.model:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        cur_list = []
        if 'MultiAction' in kw:
            kw["name"] = ""
            command = kw['MultiAction']
            cur_list = [x for x in kw if kw[x] == 'on' in kw[x]]
        else:
            obj_name = kw.get('name')
            if obj_name:
                cur_list = [ obj_name ]

        if len(cur_list) < 1:
            flash(_('No fence devices were selected'),
                status='error')
            redirect(tmpl_context.cluster_url)

        if command == 'Create':
            fret = validateNewFenceDevice(self.model, **kw)
            if fret[0] is True:
                self.model.setModified(True)
                rh.update_cluster_conf(self.model)
                log.info('User "%s" created fence devices "%s" in cluster "%s"'
                    % (self.username, ', '.join(cur_list), self.name))
            else:
                msgs = fret[1]
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
        elif command == 'Delete':
            log.info('User "%s" deleted fence devices "%s" in cluster "%s"'
                % (self.username, ', '.join(cur_list), self.name))
            flash(_('Deleting fence devices ') + ', '.join(cur_list))
            updated = False
            for i in cur_list:
                cur_fencedev = self.model.getFenceDeviceByName(i)
                if not cur_fencedev:
                    continue
                updated |= self.model.deleteFenceDevice(cur_fencedev)
            if updated:
                self.model.setModified()
                rh.update_cluster_conf(self.model)
        elif command == 'Update':
            fret = validateFenceDevice(self.model, **kw)
            if fret[0] is True:
                self.model.setModified(True)
                rh.update_cluster_conf(self.model)
                log.info('User "%s" created fence device "%s" in cluster "%s"'
                    % (self.username, ', '.join(cur_list), self.name))
            else:
                msgs = fret[1]
                if msgs and len(msgs) > 0:
                    flash(', '.join(msgs), status="error")
        else:
            log.error('User "%s" submitted unknown command "%s" for fence devices "%s" from cluster "%s"'
                % (self.username, command, ', '.join(cur_list), self.name))
            flash(_('An unknown command "%s" was given for fence devices "%s"')
                    % (command, ', '.join(cur_list)),
                status='error')
        redirect(tmpl_context.cluster_url)

    @expose("luci.templates.configure")
    def configure(self, *args):
        configure_cmd = '/cluster/%s/configure_cmd' % self.name
        return dict(use_referrer=False, page='nodes', name='configure', base_url='/cluster/' + self.name + '/configure', configure_cmd=configure_cmd)

    @expose("luci.templates.configure")
    def configure_cmd(self, command=None, *args, **kw):
        self.get_model()
        tmpl_context.cluster_url = '/cluster/%s/configure' % self.name

        if not self.model:
            flash(_('Unable to contact any nodes in this cluster'),
                status="error")
            redirect(tmpl_context.cluster_url)

        if kw['page'] == "QDisk":
            vret = vcp.validate_qdisk_prop_form(self.model, **kw)
        else:
            vret = vcp.validate_cluster_config_form(self.model, **kw)

        if vret[0] is True:
            log.info('User "%s" updated the cluster properties for cluster "%s"'
                    % (self.username, self.name))
            flash("Applying Settings")
            msgs = vret[1].get('flash')
            if msgs and len(msgs) > 0:
                flash(', '.join(msgs), status="info")
            self.model.setModified(True)
            rh.update_cluster_conf(self.model)
            if vret[1].get('start_qdisk'):
                log.info('Starting qdiskd for cluster "%s"' % self.name)
                rh.cluster_node_start(self.name, [])
        else:
            msgs = vret[1].get('errors')
            if msgs and len(msgs) > 0:
                flash(', '.join(msgs), status="error")
        redirect(tmpl_context.cluster_url)

    @expose()
    def lookup(self, nodename, *args):
        inc = IndividualNodeController(self.name, self.data, nodename)
        return inc, args

class IndividualNodeController(BaseController):
    def __init__(self, name, iccdata, nodename):
      self.name = name
      self.data = iccdata
      self.nodename = nodename
      tmpl_context.show_sidebar = True

    @expose("luci.templates.node")
    def default(self):
        db = get_cluster_db_obj(self.name)
        return dict(page='nodes', name=self.nodename, base_url='/nodes', nodes=db.nodes)

    @expose()
    def lookup(self, nodename, *args):
        icc = IndividualClusterController(self.name, self.data, nodename)
        return icc, args
