# Copyright (C) 2006-2009 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from TagObject import TagObject

TAG_NAME = "cluster"

class Cluster(TagObject):
  def __init__(self):
    TagObject.__init__(self)
    self.TAG_NAME = TAG_NAME
    self.is_cfg_version_dirty = False

  def getConfigVersion(self):
    version = self.getAttribute("config_version")
    return version

  def setConfigVersion(self, version):
    current_version = self.getConfigVersion()
    if current_version == version:
      return

    self.addAttribute("config_version", version)
    self.is_cfg_version_dirty = True

  def incrementConfigVersion(self):
    version = self.getAttribute("config_version")
    intversion = int(version)
    intversion = intversion + 1
    self.addAttribute("config_version", str(intversion))
    #self.is_cfg_version_dirty = True

  def doesClusterUseQuorumDisk(self):
    kids = self.getChildren()
    for kid in kids:
      if kid.getTagName().strip() == "quorumd":
        return True

    return False

  def getQuorumdPtr(self):
    kids = self.getChildren()
    for kid in kids:
      if kid.getTagName().strip() == "quorumd":
        return kid

    return None

  def generateXML(self, doc, parent=None):
    if self.is_cfg_version_dirty is False:
      self.incrementConfigVersion()
    else:
      self.is_cfg_version_dirty = False

    tag = doc.createElement(self.TAG_NAME)
    if parent is None:
      parent = doc
    parent.appendChild(tag)
    self.exportAttributes(tag)
    if len(self.children) > 0:
      for child in self.children:
        if child is not None:
          child.generateXML(doc, tag)

  def getNameAlias(self):
    return self.getAlias()

  def getAlias(self):
    # returns None if no alias set
    alias = self.getAttribute("alias")
    if alias is None:
      alias = self.getName()
    return alias
